@@ -57,13 +57,36 @@ module Data.Aeson
     , fromJSON
     , ToJSON(..)
     , KeyValue(..)
+    -- ** Keys for maps
+    , ToJSONKey(..)
+    , ToJSONKeyFunction(..)
+    , FromJSONKey(..)
+    , FromJSONKeyFunction(..)
+    -- ** Liftings to unary and binary type constructors
+    , FromJSON1(..)
+    , parseJSON1
+    , FromJSON2(..)
+    , parseJSON2
+    , ToJSON1(..)
+    , toJSON1
+    , toEncoding1
+    , ToJSON2(..)
+    , toJSON2
+    , toEncoding2
     -- ** Generic JSON classes and options
     , GFromJSON(..)
+    , FromArgs(..)
     , GToJSON(..)
     , GToEncoding(..)
+    , ToArgs(..)
+    , Zero
+    , One
     , genericToJSON
+    , genericLiftToJSON
     , genericToEncoding
+    , genericLiftToEncoding
     , genericParseJSON
+    , genericLiftParseJSON
     , defaultOptions
 
     -- * Inspecting @'Value's@
@@ -87,16 +110,23 @@ module Data.Aeson
     , json'
     ) where
 
-import Data.Aeson.Encode (encode)
-import Data.Aeson.Parser.Internal (decodeWith, decodeStrictWith,
-                                   eitherDecodeWith, eitherDecodeStrictWith,
-                                   jsonEOF, json, jsonEOF', json')
+import Prelude ()
+import Prelude.Compat
+
+import Data.Aeson.Types.FromJSON (ifromJSON)
+import Data.Aeson.Encoding (encodingToLazyByteString)
+import Data.Aeson.Parser.Internal (decodeWith, decodeStrictWith, eitherDecodeWith, eitherDecodeStrictWith, jsonEOF, json, jsonEOF', json')
 import Data.Aeson.Types
-import Data.Aeson.Types.Instances (ifromJSON)
 import Data.Aeson.Types.Internal (JSONPath, formatError)
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Lazy as L
 
+-- | Efficiently serialize a JSON value as a lazy 'L.ByteString'.
+--
+-- This is implemented in terms of the 'ToJSON' class's 'toEncoding' method.
+encode :: (ToJSON a) => a -> L.ByteString
+encode = encodingToLazyByteString . toEncoding
+
 -- | Efficiently deserialize a JSON value from a lazy 'L.ByteString'.
 -- If this fails due to incomplete or invalid input, 'Nothing' is
 -- returned.
@@ -136,7 +166,7 @@ decode' :: (FromJSON a) => L.ByteString -> Maybe a
 decode' = decodeWith jsonEOF' fromJSON
 {-# INLINE decode' #-}
 
--- | Efficiently deserialize a JSON value from a lazy 'L.ByteString'.
+-- | Efficiently deserialize a JSON value from a strict 'B.ByteString'.
 -- If this fails due to incomplete or invalid input, 'Nothing' is
 -- returned.
 --
