@@ -37,13 +37,43 @@ module Data.Aeson.Types
     , KeyValue(..)
     , modifyFailure
 
+    -- ** Keys for maps
+    , ToJSONKey(..)
+    , ToJSONKeyFunction(..)
+    , toJSONKeyText
+    , contramapToJSONKeyFunction
+    , FromJSONKey(..)
+    , FromJSONKeyFunction(..)
+    , fromJSONKeyCoerce
+    , coerceFromJSONKeyFunction
+    , mapFromJSONKeyFunction
+
+    -- ** Liftings to unary and binary type constructors
+    , FromJSON1(..)
+    , parseJSON1
+    , FromJSON2(..)
+    , parseJSON2
+    , ToJSON1(..)
+    , toJSON1
+    , toEncoding1
+    , ToJSON2(..)
+    , toJSON2
+    , toEncoding2
+
     -- ** Generic JSON classes
     , GFromJSON(..)
+    , FromArgs(..)
     , GToJSON(..)
     , GToEncoding(..)
+    , ToArgs(..)
+    , Zero
+    , One
     , genericToJSON
+    , genericLiftToJSON
     , genericToEncoding
+    , genericLiftToEncoding
     , genericParseJSON
+    , genericLiftParseJSON
 
     -- * Inspecting @'Value's@
     , withObject
@@ -61,6 +91,10 @@ module Data.Aeson.Types
     , (.!=)
     , object
 
+    , listEncoding
+    , listValue
+    , listParser
+
     -- * Generic and TH encoding configuration
     , Options(..)
     , SumEncoding(..)
@@ -70,7 +104,15 @@ module Data.Aeson.Types
     , defaultTaggedObject
     ) where
 
-import Data.Aeson.Encode.Functions (foldable, pairs)
-import Data.Aeson.Types.Generic ()
-import Data.Aeson.Types.Instances
+import Prelude ()
+import Prelude.Compat
+
+import Data.Aeson.Encoding (Encoding, unsafeToEncoding, fromEncoding, Series, pairs)
+import Data.Aeson.Types.Class
 import Data.Aeson.Types.Internal
+import Data.Foldable (toList)
+
+-- | Encode a 'Foldable' as a JSON array.
+foldable :: (Foldable t, ToJSON a) => t a -> Encoding
+foldable = toEncoding . toList
+{-# INLINE foldable #-}
