@@ -1,5 +1,8 @@
-{-# LANGUAGE CPP, DeriveDataTypeable, GeneralizedNewtypeDeriving, Rank2Types,
-    RecordWildCards #-}
+{-# LANGUAGE CPP #-}
+{-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE GeneralizedNewtypeDeriving #-}
+{-# LANGUAGE Rank2Types #-}
+{-# LANGUAGE RecordWildCards #-}
 #if __GLASGOW_HASKELL__ >= 800
 -- a) THQ works on cross-compilers and unregisterised GHCs
 -- b) may make compilation faster as no dynamic loading is ever needed (not sure about this)
@@ -24,14 +27,12 @@ module Data.Aeson.Types.Internal
     (
     -- * Core JSON types
       Value(..)
-    , Encoding(..)
-    , unsafeToEncoding
-    , Series(..)
     , Array
     , emptyArray, isEmptyArray
     , Pair
     , Object
     , emptyObject
+
     -- * Type conversion
     , Parser
     , Result(..)
@@ -62,36 +63,32 @@ module Data.Aeson.Types.Internal
     , DotNetTime(..)
     ) where
 
+import Prelude ()
+import Prelude.Compat
+
+import Control.Applicative (Alternative(..))
 import Control.Arrow (first)
-import Control.Applicative
 import Control.DeepSeq (NFData(..))
 import Control.Monad (MonadPlus(..), ap)
-import qualified Control.Monad.Fail as Fail
-import Data.ByteString.Builder (Builder, char7, toLazyByteString)
 import Data.Char (isLower, isUpper, toLower, isAlpha, isAlphaNum)
 import Data.Data (Data)
 import Data.Foldable (foldl')
 import Data.HashMap.Strict (HashMap)
 import Data.Hashable (Hashable(..))
-import Data.Semigroup (Semigroup((<>)))
 import Data.Scientific (Scientific)
+import Data.Semigroup (Semigroup((<>)))
 import Data.String (IsString(..))
 import Data.Text (Text, pack, unpack)
 import Data.Time (UTCTime)
 import Data.Time.Format (FormatTime)
 import Data.Typeable (Typeable)
 import Data.Vector (Vector)
+import qualified Control.Monad.Fail as Fail
 import qualified Data.HashMap.Strict as H
 import qualified Data.Scientific as S
 import qualified Data.Vector as V
 import qualified Language.Haskell.TH.Syntax as TH
 
-#if !MIN_VERSION_base(4,8,0)
-import Data.Foldable (Foldable(..))
-import Data.Monoid (Monoid(..))
-import Data.Traversable (Traversable(..))
-#endif
-
 #if !MIN_VERSION_unordered_containers(0,2,6)
 import Data.List (sort)
 #endif
@@ -338,52 +335,6 @@ data Value = Object !Object
            | Null
              deriving (Eq, Read, Show, Typeable, Data)
 
--- | An encoding of a JSON value.
-newtype Encoding = Encoding {
-      fromEncoding :: Builder
-      -- ^ Acquire the underlying bytestring builder.
-    } deriving (Semigroup,Monoid)
-
--- | Make Encoding from Builder.
---
--- Use with care! You have to make sure that the passed Builder
--- is a valid JSON Encoding!
-unsafeToEncoding :: Builder -> Encoding
-unsafeToEncoding = Encoding
-
-instance Show Encoding where
-    show (Encoding e) = show (toLazyByteString e)
-
-instance Eq Encoding where
-    Encoding a == Encoding b = toLazyByteString a == toLazyByteString b
-
-instance Ord Encoding where
-    compare (Encoding a) (Encoding b) =
-      compare (toLazyByteString a) (toLazyByteString b)
-
--- | A series of values that, when encoded, should be separated by
--- commas. Since 0.11.0.0, the '.=' operator is overloaded to create
--- either @(Text, Value)@ or 'Series'. You can use Series when
--- encoding directly to a bytestring builder as in the following
--- example:
---
--- > toEncoding (Person name age) = pairs ("name" .= name <> "age" .= age)
-data Series = Empty
-            | Value Encoding
-            deriving (Typeable)
-
-instance Semigroup Series where
-    Empty   <> a = a
-    Value a <> b =
-        Value $
-        a <> case b of
-               Empty   -> mempty
-               Value c -> Encoding (char7 ',') <> c
-
-instance Monoid Series where
-    mempty  = Empty
-    mappend = (<>)
-
 -- | A newtype wrapper for 'UTCTime' that uses the same non-standard
 -- serialization format as Microsoft .NET, whose
 -- <https://msdn.microsoft.com/en-us/library/system.datetime(v=vs.110).aspx System.DateTime>
@@ -482,7 +433,7 @@ parseEither m v = runParser (m v) [] onError Right
 -- | Annotate an error message with a
 -- <http://goessner.net/articles/JsonPath/ JSONPath> error location.
 formatError :: JSONPath -> String -> String
-formatError path msg = "Error in " ++ (format "$" path) ++ ": " ++ msg
+formatError path msg = "Error in " ++ format "$" path ++ ": " ++ msg
   where
     format :: String -> JSONPath -> String
     format pfx []                = pfx
@@ -601,6 +552,20 @@ data SumEncoding =
     -- by the encoded value of that field! If the constructor is not a
     -- record the encoded constructor contents will be stored under
     -- the 'contentsFieldName' field.
+  | UntaggedValue
+    -- ^ Constructor names won't be encoded. Instead only the contents of the
+    -- constructor will be encoded as if the type had single constructor. JSON
+    -- encodings have to be disjoint for decoding to work properly.
+    --
+    -- When decoding, constructors are tried in the order of definition. If some
+    -- encodings overlap, the first one defined will succeed.
+    --
+    -- /Note:/ Nullary constructors are encoded as the string (using
+    -- 'constructorTagModifier'). Having a nullary constructor alongside a
+    -- single field constructor that encodes to a string leads to ambiguity.
+    --
+    -- /Note:/ Only the last error is kept when decoding, so in the case of
+    -- mailformed JSON, only an error for the last constructor will be reported.
   | ObjectWithSingleField
     -- ^ A constructor will be encoded to an object with a single
     -- field named after the constructor tag (modified by the
