@@ -1,4 +1,10 @@
-{-# LANGUAGE BangPatterns, OverloadedStrings #-}
+{-# LANGUAGE BangPatterns #-}
+{-# LANGUAGE OverloadedStrings #-}
+
+module Main (main) where
+
+import Prelude ()
+import Prelude.Compat
 
 import Control.DeepSeq
 import Control.Exception
