@@ -1,10 +1,23 @@
+{-# LANGUAGE CPP #-}
+{-# LANGUAGE PackageImports #-}
+
+{-# OPTIONS_GHC -fno-warn-unused-imports #-}
+
 module Typed.Common (load) where
 
-import Data.Aeson hiding (Result)
+import Prelude ()
+import Prelude.Compat
+
 import Data.ByteString.Lazy as L
-import Control.Applicative
-import System.IO
 import System.Exit
+import System.IO
+
+#ifndef HAS_BOTH_AESON_AND_BENCHMARKS
+import Data.Aeson hiding (Result)
+#else
+import "aeson" Data.Aeson hiding (Result)
+import qualified "aeson-benchmarks" Data.Aeson as B
+#endif
 
 load :: FromJSON a => FilePath -> IO a
 load fileName = do
