@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE DeriveGeneric #-}
 {-# LANGUAGE FlexibleInstances #-}
@@ -7,11 +8,20 @@
 
 module Types where
 
-import qualified Data.Map as Map
+import Prelude ()
+import Prelude.Compat
+
 import Data.Data
+import Data.Functor.Compose (Compose (..))
+import Data.Functor.Identity (Identity (..))
 import Data.Text
 import GHC.Generics
 import Test.QuickCheck (Property, counterexample)
+import qualified Data.Map as Map
+
+type I = Identity
+type Compose3  f g h = Compose (Compose f g) h
+type Compose3' f g h = Compose f (Compose g h)
 
 data Foo = Foo {
       fooInt :: Int
@@ -56,7 +66,29 @@ data SomeType a = Nullary
                 | Record { testOne   :: Double
                          , testTwo   :: Maybe Bool
                          , testThree :: Maybe a
-                         } deriving (Eq, Show)
+                         }
+                | List [a]
+  deriving (Eq, Show)
+
+-- | This type requires IncoherentInstances for the instances of the type
+-- classes Data.Aeson.TH.LookupField and Data.Aeson.Types.FromJSON.FromRecord.
+--
+-- The minimum known requirements for this type are:
+-- * Record type with at least two fields
+-- * One field type is either a type parameter or a type/data family
+-- * Another field type is a @Maybe@ of the above field type
+data IncoherentInstancesNeeded a = IncoherentInstancesNeeded
+  { incoherentInstancesNeededMaybeNot :: a
+  , incoherentInstancesNeededMaybeYes :: Maybe a
+  } deriving Generic
+
+-- Used for testing UntaggedValue SumEncoding
+data EitherTextInt
+    = LeftBool Bool
+    | RightInt Int
+    | BothTextInt Text Int
+    | NoneNullary
+    deriving (Eq, Show)
 
 data GADT a where
     GADT :: { gadt :: String } -> GADT String
@@ -75,6 +107,10 @@ deriving instance Generic (Sum4 a b c d)
 deriving instance Generic (Approx a)
 deriving instance Generic Nullary
 deriving instance Generic (SomeType a)
+#if __GLASGOW_HASKELL__ >= 706
+deriving instance Generic1 SomeType
+#endif
+deriving instance Generic EitherTextInt
 
 failure :: Show a => String -> String -> a -> Property
 failure func msg v = counterexample
